@@ -38,6 +38,8 @@
 #include <geoclue/gc-iface-position.h>
 #include <geoclue/gc-iface-address.h>
 
+#include "connectivity.h"
+
 #define GEOCLUE_DBUS_SERVICE_PLAZES "org.freedesktop.Geoclue.Providers.Plazes"
 #define GEOCLUE_DBUS_PATH_PLAZES "/org/freedesktop/Geoclue/Providers/Plazes"
 #define PLAZES_URL "http://plazes.com/suggestions.xml"
@@ -51,6 +53,7 @@
 typedef struct _GeocluePlazes {
 	GcProvider parent;
 	GMainLoop *loop;
+	GeoclueConnectivity *conn;
 	GcWebService *web_service;
 	GeoclueStatus last_status;
 } GeocluePlazes;
@@ -98,96 +101,17 @@ geoclue_plazes_set_status (GeocluePlazes *self, GeoclueStatus status)
 		gc_iface_geoclue_emit_status_changed (GC_IFACE_GEOCLUE (self), status);
     }
 }
-		
-/* Parse /proc/net/route to get default gateway address and then parse
- * /proc/net/arp to find matching mac address. 
- *
- * There are some problems with this. First, it's IPv4 only.
- * Second, there must be a way to do this with ioctl, but that seemed really
- * complicated... even /usr/sbin/arp parses /proc/net/arp 
- * 
- * returns:
- *   1 : on success
- *   0 : no success, no errors
- *  <0 : error
- */
-int
-get_mac_address (char **mac)
+
+static char *
+mac_strdown (char *mac)
 {
-	char *content;
-	char **lines, **entry;
-	GError *error = NULL;
-	char *route_gateway = NULL;
-
-	g_assert (*mac == NULL);
-
-	if (!g_file_get_contents ("/proc/net/route", &content, NULL, &error)) {
-		g_warning ("Failed to read /proc/net/route: %s", error->message);
-		g_error_free (error);
-		return -1;
+	guint i;
+	for (i = 0; mac[i] != '\0' ; i++) {
+		if (g_ascii_isalpha (mac[i]))
+			mac[i] = g_ascii_tolower (mac[i]);
 	}
-
-	lines = g_strsplit (content, "\n", 0);
-	g_free (content);
-	entry = lines + 1;
-
-	while (*entry && strlen (*entry) > 0) {
-		char dest[9];
-		char gateway[9];
-		if (sscanf (*entry, 
-			        "%*s %8[0-9A-Fa-f] %8[0-9A-Fa-f] %*s", 
-			        dest, gateway) != 2) {
-			g_warning ("Failed to parse /proc/net/route entry '%s'", *entry);
-		} else if (strcmp (dest, "00000000") == 0) {
-			route_gateway = g_strdup (gateway);
-			break;
-		}
-		entry++;
-	}
-	g_strfreev (lines);	
-
-	if (!route_gateway) {
-		g_warning ("Failed to find default route in /proc/net/route");
-		return -1;
-	}
-
-	if (!g_file_get_contents ("/proc/net/arp", &content, NULL, &error)) {
-		g_warning ("Failed to read /proc/net/arp: %s", error->message);
-		g_error_free (error);
-		return -1;
-	}
-	
-	lines = g_strsplit (content, "\n", 0);
-	g_free (content);
-	entry = lines+1;
-	while (*entry && strlen (*entry) > 0) {
-		char hwa[100];
-		char *arp_gateway;
-		int ip[4];
-		
-		if (sscanf(*entry, 
-		           "%d.%d.%d.%d 0x%*x 0x%*x %100s %*s %*s\n", 
-		           &ip[0], &ip[1], &ip[2], &ip[3], hwa) != 5) {
-			g_warning ("Failed to parse /proc/net/arp entry '%s'", *entry);
-		} else {
-			arp_gateway = g_strdup_printf ("%02X%02X%02X%02X", ip[3], ip[2], ip[1], ip[0]);
-			if (strcmp (arp_gateway, route_gateway) == 0) {
-				g_free (arp_gateway);
-				*mac = g_strdup (hwa);
-				break;
-			}
-			g_free (arp_gateway);
-			
-		}
-		entry++;
-	}
-	g_free (route_gateway);
-	g_strfreev (lines);
-
-	return *mac ? 1 : 0;
+	return mac;
 }
-
-
 /* Position interface implementation */
 
 static gboolean 
@@ -201,8 +125,7 @@ geoclue_plazes_get_position (GcIfacePosition        *iface,
                              GError                **error)
 {
 	GeocluePlazes *plazes;
-	int i, ret_val;
-	char *mac = NULL;
+	char *mac, *mac_lc;
 	
 	plazes = (GEOCLUE_PLAZES (iface));
 	
@@ -211,17 +134,7 @@ geoclue_plazes_get_position (GcIfacePosition        *iface,
 		*timestamp = time (NULL);
 	}
 
-	/* we may be trying to read /proc/net/arp right after network connection. 
-	 * It's sometimes not up yet, try a couple of times */
-	for (i = 0; i < 5; i++) {
-		ret_val = get_mac_address (&mac);
-		if (ret_val < 0)
-			return FALSE;
-		else if (ret_val == 1)
-			break;
-		usleep (200);
-	}
-
+	mac = geoclue_connectivity_get_router_mac (plazes->conn);
 	if (mac == NULL) {
 		g_set_error (error, GEOCLUE_ERROR, 
 		             GEOCLUE_ERROR_NOT_AVAILABLE, 
@@ -229,21 +142,27 @@ geoclue_plazes_get_position (GcIfacePosition        *iface,
 		geoclue_plazes_set_status (plazes, GEOCLUE_STATUS_ERROR);
 		return FALSE;
 	}
-	
-    geoclue_plazes_set_status (plazes, GEOCLUE_STATUS_ACQUIRING);
 
+	mac = mac_strdown (mac);
+
+	geoclue_plazes_set_status (plazes, GEOCLUE_STATUS_ACQUIRING);
+
+	mac_lc = g_ascii_strdown (mac, -1);
+	g_free (mac);
 	if (!gc_web_service_query (plazes->web_service, error,
-	                           PLAZES_KEY_MAC, mac, 
+	                           PLAZES_KEY_MAC, mac,
 	                           (char *)0)) {
-		g_free (mac);
+		g_free (mac_lc);
         // did not get a reply; we can try again later
 		geoclue_plazes_set_status (plazes, GEOCLUE_STATUS_AVAILABLE);
-		g_set_error (error, GEOCLUE_ERROR, 
-		             GEOCLUE_ERROR_NOT_AVAILABLE, 
+		g_set_error (error, GEOCLUE_ERROR,
+		             GEOCLUE_ERROR_NOT_AVAILABLE,
 		             "Did not get reply from server");
 		return FALSE;
 	}
-	
+
+	g_free (mac);
+
 	if (latitude && gc_web_service_get_double (plazes->web_service, 
 	                                           latitude, PLAZES_LAT_XPATH)) {
 		*fields |= GEOCLUE_POSITION_FIELDS_LATITUDE;
@@ -288,8 +207,7 @@ geoclue_plazes_get_address (GcIfaceAddress   *iface,
 {
 	
 	GeocluePlazes *plazes = GEOCLUE_PLAZES (iface);
-	int i, ret_val;
-	char *mac = NULL;
+	char *mac, *mac_lc;
 
 	GeoclueAccuracyLevel level = GEOCLUE_ACCURACY_LEVEL_NONE;
 	
@@ -297,16 +215,7 @@ geoclue_plazes_get_address (GcIfaceAddress   *iface,
 		*timestamp = time (NULL);
 	}
 
-	/* we may be trying to read /proc/net/arp right after network connection. 
-	 * It's sometimes not up yet, try a couple of times */
-	for (i = 0; i < 5; i++) {
-		ret_val = get_mac_address (&mac);
-		if (ret_val < 0)
-			return FALSE;
-		else if (ret_val == 1)
-			break;
-		usleep (200);
-	}
+	mac = geoclue_connectivity_get_router_mac (plazes->conn);
 
 	if (mac == NULL) {
 		g_set_error (error, GEOCLUE_ERROR, 
@@ -318,17 +227,20 @@ geoclue_plazes_get_address (GcIfaceAddress   *iface,
 	
     geoclue_plazes_set_status (plazes, GEOCLUE_STATUS_ACQUIRING);
 
+	mac_lc = g_ascii_strdown (mac, -1);
+	g_free (mac);
 	if (!gc_web_service_query (plazes->web_service, error,
-	                           PLAZES_KEY_MAC, mac, 
+	                           PLAZES_KEY_MAC, mac_lc, 
 	                           (char *)0)) {
-		g_free (mac);
+		g_free (mac_lc);
 		geoclue_plazes_set_status (plazes, GEOCLUE_STATUS_AVAILABLE);
 		g_set_error (error, GEOCLUE_ERROR, 
 		             GEOCLUE_ERROR_NOT_AVAILABLE, 
 		             "Did not get reply from server");
 		return FALSE;
 	}
-	
+	g_free (mac_lc);
+
 	if (address) {
 		char *str;
 		
@@ -397,7 +309,11 @@ static void
 geoclue_plazes_finalize (GObject *obj)
 {
 	GeocluePlazes *plazes = GEOCLUE_PLAZES (obj);
-	
+
+	if (plazes->conn != NULL) {
+		g_object_unref (plazes->conn);
+		plazes->conn = NULL;
+	}
 	g_object_unref (plazes->web_service);
 	
 	((GObjectClass *) geoclue_plazes_parent_class)->finalize (obj);
@@ -425,7 +341,8 @@ geoclue_plazes_init (GeocluePlazes *plazes)
 	                         GEOCLUE_DBUS_SERVICE_PLAZES,
 	                         GEOCLUE_DBUS_PATH_PLAZES,
 	                         "Plazes", "Plazes.com based provider, uses gateway mac address to locate");
-	
+
+	plazes->conn = geoclue_connectivity_new ();
 	plazes->web_service = g_object_new (GC_TYPE_WEB_SERVICE, NULL);
 	gc_web_service_set_base_url (plazes->web_service, PLAZES_URL);
     geoclue_plazes_set_status (plazes, GEOCLUE_STATUS_AVAILABLE);
